import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import com.mindfusion.diagramming.AttachToNode;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.InteractionState;
import com.mindfusion.diagramming.RenderOptions;
import com.mindfusion.diagramming.Pen;


public class AspectNode extends DiagramNode
{
	public AspectNode(CauseNode causeNode)
	{
		super(causeNode.getParent());
		this.causeNode = causeNode;
		setPen(new Pen(1, Color.blue));
	}
	
	@Override
	protected void draw(Graphics2D g, RenderOptions options)
	{
		int code = (int)'a' + (getZIndex() == 0 ? getParent().getItems().size() : getZIndex()) - 1;
		char c = (char)(code);

		float x = getBounds().x;
		float y = getBounds().y;
		
		Point2D.Float p1 = new Point2D.Float(x, y);
		p1.x += getBounds().width  / 2;
		
		Point2D.Float p2 = (Point2D.Float)p1.clone();
		p2.y += getBounds().height;
		
		// draw the aspect line
		Paint p = g.getPaint();
		Font f = g.getFont();
		g.setPaint(getPen().getColor());
		g.setStroke(new BasicStroke(getPen().getWidth()));
		g.draw(new Line2D.Float(p1.x, p1.y, p2.x, p2.y));
		
		// find the intersection point with the cause line
		double m = (causeNode.getP2().getY() - causeNode.getP1().getY()) /
			(causeNode.getP2().getX() - causeNode.getP1().getX());
		double b = causeNode.getP2().getY() - m*causeNode.getP2().getX();
		double yP = m*p1.getX() + b;

		// draw the label
		if (Math.abs(p1.getY() - yP) > Math.abs(p2.getY() - yP))
		{
			g.setFont(new Font("Trebuchet MS", Font.BOLD, 5));
			g.setPaint(getPen().getColor());
			g.drawString(String.valueOf(c), (float)(p1.x - 1.7), (float)(p1.y - 1));
		}
		else
		{
			g.setFont(new Font("Trebuchet MS", Font.BOLD, 5));
			g.setPaint(getPen().getColor());
			g.drawString(String.valueOf(c), (float)p2.x - 1.7f, (float)p2.y + 5);
		}
		
		g.setFont(f);
		g.setPaint(p);
	}

	protected void startCreate(Point2D org)
	{
		super.startCreate(org);
		start = org;
	}
	
	protected void updateCreate(Point2D current)
	{
		super.updateCreate(current);

		double dy = start.getY() - current.getY();

		setBounds(MainFrame.fromLTRB(start.getX() - 0.5f, start.getY(),
				start.getX() + 0.5f, start.getY() - dy));
	}

	protected boolean allowCreate(Point2D current, InteractionState ist)
	{
		return true;
	}
	
	protected void completeCreate(Point2D end)
	{
		super.completeCreate(end);
		attachTo(causeNode, AttachToNode.TopLeft);
		causeNode.getSubordinateGroup().setAutoDeleteItems(true);	
	}
	
	@Override
	protected void drawShadow(Graphics2D g, RenderOptions options)
	{
	}
	
	@Override
	protected Rectangle2D getRepaintRect(boolean includeConnected)
	{
		Rectangle2D rect = super.getRepaintRect(includeConnected);
		rect.setRect(rect.getX(), rect.getY() - 5, rect.getWidth(),	rect.getHeight() + 10);
		return rect;
	}

	public Rectangle2D getEditRect()
	{
		return null;
	}

	public String getTextToEdit()
	{
		return null;
	}

	public void setEditedText(String newText)
	{
	}

	public Rectangle2D getEditRect(Point2D mousePosition)
	{
		return null;
	}
	
	CauseNode causeNode = null;
	Point2D start;
}
