import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.SpringLayout;

import com.mindfusion.diagramming.Behavior;
import com.mindfusion.diagramming.Brush;
import com.mindfusion.diagramming.Diagram;
import com.mindfusion.diagramming.DiagramNode;
import com.mindfusion.diagramming.DiagramView;
import com.mindfusion.diagramming.FractalLayout;
import com.mindfusion.diagramming.GradientBrush;
import com.mindfusion.diagramming.Shape;
import com.mindfusion.diagramming.ShapeNode;
import com.mindfusion.diagramming.SolidBrush;

public class FractalLayoutFrame extends JFrame
{
	public FractalLayoutFrame()
	{
		super("MindFusion.Diagramming Sample: Fractal Layout");
		diagram = new Diagram();
		diagramView = new DiagramView(diagram);
		
		JScrollPane scrollPane = new JScrollPane(diagramView);
		JButton button = new JButton("Random tree");
		button.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				FractalLayoutFrame.this.randomTree();
			}
		});
		
		diagramView.addMouseWheelListener(new MouseWheelListener()
		{
			public void mouseWheelMoved(MouseWheelEvent e) {
				diagramView.setZoomFactor(diagramView.getZoomFactor() - e.getWheelRotation());
			}
		});
		
		Container cp = getContentPane();
		cp.add(scrollPane);
		cp.add(button);
		
		SpringLayout layout = new SpringLayout();
		layout.putConstraint(SpringLayout.EAST, scrollPane, 0, SpringLayout.EAST, cp);
		layout.putConstraint(SpringLayout.WEST, scrollPane, 0, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.SOUTH, scrollPane, -40, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.NORTH, scrollPane, 0, SpringLayout.NORTH, cp);
		
		layout.putConstraint(SpringLayout.WEST, button, 5, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.EAST, button, 150, SpringLayout.WEST, cp);
		layout.putConstraint(SpringLayout.SOUTH, button, -5, SpringLayout.SOUTH, cp);
		layout.putConstraint(SpringLayout.NORTH, button, -35, SpringLayout.SOUTH, cp);
		
		this.setLayout(layout);
		
		Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(size);

		this.addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});
		
		init();
	}
	
	private void init()
	{
		diagram.setLinkHeadShape(null);
		diagram.setDefaultShape(Shape.fromId("Ellipse"));
		diagram.setBackBrush(new SolidBrush(Color.white));

		//diagramView.MouseWheel += OnMouseWheel;
		diagramView.setBehavior(Behavior.Pan);

		// use differect brush for each tree level
		brushes = new Brush[]
		{
			new GradientBrush(new Color(176, 196, 222), new Color(138, 43, 226), 40),
			new GradientBrush(Color.white, new Color(173, 216, 230), 40),
			new GradientBrush(Color.white, new Color(0, 191, 255), 40),
			new GradientBrush(new Color(50, 205, 50), Color.green, 40)
		};
	}
	
	private void randomTree()
	{
		diagram.clearAll();

		ShapeNode root = diagram.getFactory().createShapeNode(nodeBounds);
		randomTree(root, 5, 4);
		arrange(root);

		// the focused control also receives MouseWheel events
		diagramView.setFocusable(true);
	}
	
	private void arrange(DiagramNode root)
	{
		FractalLayout layout = new FractalLayout();
		layout.setRoot(root);
		layout.arrange(diagram);

		diagram.resizeToFitItems(5);
		diagramView.zoomToFit();
	}

	void randomTree(DiagramNode node, int depth, int minChildren)
	{
		if (depth <= 0)
			return;

		Diagram diagram = node.getParent();
		int children = random.nextInt(depth) - 1 + minChildren;

		if (diagram.getNodes().size() < 3 && children < 2)
			children = 2;

		for (int i = 0; i < children; ++i)
		{
			// create child node and link
			ShapeNode child = diagram.getFactory().createShapeNode(nodeBounds);
			child.setBrush(brushes[depth % brushes.length]);
			diagram.getFactory().createDiagramLink(node, child);

			// build child branch
			randomTree(child, depth - 1, minChildren);
		}
	}
	
	private Random random = new Random();
	private Rectangle2D nodeBounds = new Rectangle2D.Float(0, 0, 20, 20);
	private Brush[] brushes;
	private Diagram diagram;
	private DiagramView diagramView;
}
