
package com.mindfusion.diagramming.samples.demo;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;


public class AddFieldDialog extends JDialog
{
	/**
	 * Initializes a new instance of the AddFieldDialog class with
	 * the specified parent. The dialog is centered within
	 * the parent.
	 */
	public AddFieldDialog(JFrame parent)
	{
		super(parent, true);

		_accepted = false;

		setTitle("Add Field");
		setSize(265, 150);
		setResizable(false);

		// Center in parent
		Rectangle parentBounds = new Rectangle();
		parent.getBounds(parentBounds);

		setLocation((parentBounds.width - getWidth()) / 2,
			(parentBounds.height - getHeight()) / 2);

		// Initialize frame
		initializeComponent();
	}

	/**
	 * Initializes child controls.
	 */
	private void initializeComponent()
	{
		setLayout(null);

		//
		// _lblFieldName
		//
		_lblFieldName = new JLabel();
		_lblFieldName.setText("Field name:");
		_lblFieldName.setLocation(7, 14);
		_lblFieldName.setSize(66, 14);

		//
		// _txtFieldName
		//
		_txtFieldName = new JTextField();
		_txtFieldName.setLocation(80, 12);
		_txtFieldName.setSize(173, 20);

		//
		// _lblFieldType
		//
		_lblFieldType = new JLabel();
		_lblFieldType.setText("Field type:");
		_lblFieldType.setLocation(7, 42);
		_lblFieldType.setSize(66, 13);

		//
		// _cbFieldType
		//
		_cbFieldType = new JComboBox();
		_cbFieldType.setModel(
			new DefaultComboBoxModel(new Object[]
			{
				"Text",
				"Number",
				"Date/Time",
				"Auto number",
				"Yes/No"
			}));
		_cbFieldType.setLocation(80, 38);
		_cbFieldType.setSize(173, 21);

		//
		// _btnAdd
		//
		_btnAdd = new JButton();
		_btnAdd.setText("Add");
		_btnAdd.setLocation(45, 90);
		_btnAdd.setSize(80, 20);
		_btnAdd.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					AddFieldDialog.this.onBtnAdd(e);
				}
			}
		);

		//
		// _btnCancel
		//
		_btnCancel = new JButton();
		_btnCancel.setText("Cancel");
		_btnCancel.setLocation(136, 90);
		_btnCancel.setSize(80, 20);
		_btnCancel.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					AddFieldDialog.this.onBtnCancel(e);
				}
			}
		);

		Container contents = getContentPane();

		contents.add(_lblFieldName);
		contents.add(_txtFieldName);
		contents.add(_lblFieldType);
		contents.add(_cbFieldType);
		contents.add(_btnAdd);
		contents.add(_btnCancel);
	}

	/**
	 * Handles the Add button click.
	 */
	private void onBtnAdd(ActionEvent e)
	{
		_accepted = true;

		hide();
	}

	/**
	 * Handles the Cancel button click.
	 */
	private void onBtnCancel(ActionEvent e)
	{
		_accepted = false;

		hide();
	}

	/**
	 * Gets whether the user has confirmed the dialog.
	 */
	public boolean getAccepted() { return _accepted; }

	public String getFieldName() { return _txtFieldName.getText(); }

	public void setFieldName(String value) { _txtFieldName.setText(value); }

	public int getFieldType()
	{
		return _cbFieldType.getSelectedIndex();
	}

	public void setFieldType(int value)
	{
		_cbFieldType.setSelectedIndex(value);
	}

	public String getFieldTypeText()
	{
		return _shortType[_cbFieldType.getSelectedIndex()];
	}

	public void setFieldTypeText(String value)
	{
		_cbFieldType.setSelectedItem(null);
		for (int i = 0; i < _shortType.length; i++)
		{
			if (_shortType[i].equals(value))
			{
				_cbFieldType.setSelectedIndex(i);
				break;
			}
		}
	}

	public void setAcceptBtnText(String value)
	{
		_btnAdd.setText(value);
	}


	private static String[] _shortType =
	{
		"Text",
		"Num",
		"Date",
		"Auto",
		"Bool"
	};

	private boolean _accepted;

	private JLabel _lblFieldName;
	private JTextField _txtFieldName;
	private JLabel _lblFieldType;
	private JComboBox _cbFieldType;
	private JButton _btnAdd;
	private JButton _btnCancel;
}
