import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTextArea;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.mindfusion.diagramming.*;

public class Routing extends JFrame
{
	public Routing()
	{
		super("MindFusion.Diagramming sample: Link Routing");
		initFrame();
		initDiagram();
	}

	private void initDiagram()
	{
		diag.setBackBrush(new SolidBrush(Color.white));
		diag.getShapeNodeStyle().setBrush(
			new GradientBrush(Color.white, new Color(150, 220, 250), 30));
		diag.setSelectAfterCreate(false);
		diag.setLinkSegments((short)3);
		diag.setLinkShape(LinkShape.Cascading);		
		createObstacles();
	}
	
	private void createObstacles()
	{
		int x, y;
		for (y = 0; y < 10; y++)
		{
			int xOffset;
			xOffset = (y % 2) * 16;

			for (x = 0; x < 10; x++)
			{
				diag.getFactory().createShapeNode(10 + xOffset, 10 + y * 32, (10 + (x % 2) * 10), 10);

				xOffset = xOffset + 32;
				xOffset = xOffset + (x % 2) * 10;
			}
		}

		// GridRouter is slower, but allows customization of 
		// the routing function through various cost attributes
		diag.setLinkRouter(new GridRouter());
		diag.setRouteLinks(true);

		// route links while they are being created
		diag.getRoutingOptions().setTriggerRerouting(
			RerouteLinks.WhileCreating | RerouteLinks.WhenModified);

		diag.resizeToFitItems(10);
		diagView.setAutoScroll(true);
	}
	
	private void initFrame()
	{
		Dimension frameSize = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(frameSize);

		diag = new Diagram();
		diagView = new DiagramView(diag);
		fcScrollPane = new JScrollPane(diagView);

		textBox1 = new JTextArea();
		textAreaScrollPane = new JScrollPane(textBox1);

		optionPanel = new JPanel();

		nodeVicSizeSl = new JSlider();
		nodeVicSizeLb = new JLabel();
		nodeVicCostSl = new JSlider();
		nodeVicCostLb = new JLabel();
		lengthCostSl = new JSlider();
		lengthCostLb = new JLabel();
		crossingCostSl = new JSlider();
		crossingCostLb = new JLabel();
		turnCostLb = new JLabel();
		turnCostSl = new JSlider();

		rerouteAllArrowsBtn = new JButton();

		String text =
			"This example lets you experiment with the various arrow routing options. " +
			"Move the sliders to alter a routing parameter value and draw arrows in the flowchart " +
			"canvas on the right to see the effect of the new value. Each cost property is a " +
			"penalty value added by the route evaluation algorithm to a route cost when a given condition is met. " +
			"The turn cost is added if an arrow makes a turn when following the considered route. " +
			"Crossing cost is added if an arrow crosses another arrow. Length cost is added for each " +
			"piece of the routing grid an arrow passes through. Node vicinity cost is added " +
			"to the total cost if the arrow passes near a node.";

		// textAreaScrollPane
		textAreaScrollPane.setName("textAreaScrollPane");

		// textBox1
		textBox1.setText(text);
		textBox1.setWrapStyleWord(true);
		textBox1.setLineWrap(true);

		// optionPanel
		optionPanel.setName("optionPanel");
		optionPanel.setBackground(new Color(245, 245, 245));
		optionPanel.setLayout(null);
		optionPanel.add(turnCostLb);
		optionPanel.add(turnCostSl);
		optionPanel.add(crossingCostLb);
		optionPanel.add(crossingCostSl);
		optionPanel.add(lengthCostLb);
		optionPanel.add(lengthCostSl);
		optionPanel.add(nodeVicCostLb);
		optionPanel.add(nodeVicCostSl);
		optionPanel.add(nodeVicSizeLb);
		optionPanel.add(nodeVicSizeSl);
		optionPanel.add(rerouteAllArrowsBtn);

		// turnCostLb
		turnCostLb.setName("turnCostLb");
		turnCostLb.setBounds(5, 5, 190, 20);
		turnCostLb.setText("Turn cost: " + 15);

		// turnCostSl
		turnCostSl.setName("turnCostSl");
		turnCostSl.setBackground(new Color(245, 245, 245));
		turnCostSl.setBounds(5, 25, 190, 35);
		turnCostSl.setSnapToTicks(true);
		turnCostSl.setMaximum(50);
		turnCostSl.setMinimum(0);
		turnCostSl.setMajorTickSpacing(10);
		turnCostSl.setMinorTickSpacing(5);
		turnCostSl.setPaintTrack(true);
		turnCostSl.setPaintTicks(true);
		turnCostSl.setValue(15);
		turnCostSl.addChangeListener
		(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					turnCostTrB_Scroll();
				}
			}
		);

		// crossingCostLb
		crossingCostLb.setName("crossingCostLb");
		crossingCostLb.setBounds(5, 75, 190, 20);
		crossingCostLb.setText("Crossing cost: " + 15);

		// crossingCostSl
		crossingCostSl.setName("crossingCostSl");
		crossingCostSl.setBackground(new Color(245, 245, 245));
		crossingCostSl.setBounds(5, 95, 190, 35);
		crossingCostSl.setSnapToTicks(true);
		crossingCostSl.setMaximum(50);
		crossingCostSl.setMinimum(0);
		crossingCostSl.setMajorTickSpacing(10);
		crossingCostSl.setMinorTickSpacing(5);
		crossingCostSl.setPaintTrack(true);
		crossingCostSl.setPaintTicks(true);
		crossingCostSl.setValue(15);
		crossingCostSl.addChangeListener
		(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					crossingCostTrB_Scroll();
				}
			}
		);

		// lengthCostLb
		lengthCostLb.setName("lengthCostLb");
		lengthCostLb.setBounds(5, 145, 190, 20);
		lengthCostLb.setText("Length cost: " + 15);

		// lengthCostSl
		lengthCostSl.setName("lengthCostSl");
		lengthCostSl.setBackground(new Color(245, 245, 245));
		lengthCostSl.setBounds(5, 165, 190, 35);
		lengthCostSl.setSnapToTicks(true);
		lengthCostSl.setMaximum(50);
		lengthCostSl.setMinimum(0);
		lengthCostSl.setMajorTickSpacing(10);
		lengthCostSl.setMinorTickSpacing(5);
		lengthCostSl.setPaintTrack(true);
		lengthCostSl.setPaintTicks(true);
		lengthCostSl.setValue(15);
		lengthCostSl.addChangeListener
		(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					lengthCostTrB_Scroll();
				}
			}
		);

		// nodeVicCostLb
		nodeVicCostLb.setName("nodeVicCostLb");
		nodeVicCostLb.setBounds(5, 215, 190, 20);
		nodeVicCostLb.setText("Node visinity cost: " + 15);

		// nodeVicCostSl
		nodeVicCostSl.setName("nodeVicCostSl");
		nodeVicCostSl.setBackground(new Color(245, 245, 245));
		nodeVicCostSl.setBounds(5, 235, 190, 35);
		nodeVicCostSl.setSnapToTicks(true);
		nodeVicCostSl.setMaximum(50);
		nodeVicCostSl.setMinimum(0);
		nodeVicCostSl.setMajorTickSpacing(10);
		nodeVicCostSl.setMinorTickSpacing(5);
		nodeVicCostSl.setPaintTrack(true);
		nodeVicCostSl.setPaintTicks(true);
		nodeVicCostSl.setValue(15);
		nodeVicCostSl.addChangeListener
		(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					nodeVicCostTrB_Scroll();
				}
			}
		);

		// nodeVicSizeLb
		nodeVicSizeLb.setName("nodeVicSizeLb");
		nodeVicSizeLb.setBounds(5, 285, 190, 20);
		nodeVicSizeLb.setText("Node vicinity size: " + 15);

		// nodeVicSizeSl
		nodeVicSizeSl.setName("nodeVicSizeSl");
		nodeVicSizeSl.setBackground(new Color(245, 245, 245));
		nodeVicSizeSl.setBounds(5, 305, 190, 35);
		nodeVicSizeSl.setSnapToTicks(true);
		nodeVicSizeSl.setMaximum(50);
		nodeVicSizeSl.setMinimum(0);
		nodeVicSizeSl.setMajorTickSpacing(10);
		nodeVicSizeSl.setMinorTickSpacing(5);
		nodeVicSizeSl.setPaintTrack(true);
		nodeVicSizeSl.setPaintTicks(true);
		nodeVicSizeSl.setValue(15);
		nodeVicSizeSl.addChangeListener
		(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent e) 
				{
					nodeVicSizeTrB_Scroll();
				}
			}
		);

		// rerouteAllArrowsBtn
		rerouteAllArrowsBtn.setName("rerouteAllArrowsBtn");
		rerouteAllArrowsBtn.setText("Route All");
		rerouteAllArrowsBtn.setBounds(25, 375, 150, 25);
		rerouteAllArrowsBtn.addActionListener
		(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					rerouteAllArrowsBtn_Click();
				}
			}
		);

		Container cp = this.getContentPane();

		cp.add(fcScrollPane);
		cp.add(optionPanel);
		cp.add(textAreaScrollPane);

		javax.swing.SpringLayout layout = new javax.swing.SpringLayout();
		layout.putConstraint(javax.swing.SpringLayout.EAST, optionPanel, 0, javax.swing.SpringLayout.EAST, cp);
		layout.putConstraint(javax.swing.SpringLayout.WEST, optionPanel, -200, javax.swing.SpringLayout.EAST, cp);
		layout.putConstraint(javax.swing.SpringLayout.SOUTH, optionPanel, 0, javax.swing.SpringLayout.SOUTH, cp);
		layout.putConstraint(javax.swing.SpringLayout.NORTH, optionPanel, 0, javax.swing.SpringLayout.NORTH, cp);

		layout.putConstraint(javax.swing.SpringLayout.EAST, textAreaScrollPane, 0, javax.swing.SpringLayout.WEST, optionPanel);
		layout.putConstraint(javax.swing.SpringLayout.WEST, textAreaScrollPane, 0, javax.swing.SpringLayout.WEST, cp);
		layout.putConstraint(javax.swing.SpringLayout.SOUTH, textAreaScrollPane, 0, javax.swing.SpringLayout.SOUTH, cp);
		layout.putConstraint(javax.swing.SpringLayout.NORTH, textAreaScrollPane, -80, javax.swing.SpringLayout.SOUTH, cp);

		layout.putConstraint(javax.swing.SpringLayout.EAST, fcScrollPane, 0, javax.swing.SpringLayout.WEST, optionPanel);
		layout.putConstraint(javax.swing.SpringLayout.WEST, fcScrollPane, 0, javax.swing.SpringLayout.WEST, cp);
		layout.putConstraint(javax.swing.SpringLayout.SOUTH, fcScrollPane, 0, javax.swing.SpringLayout.NORTH, textAreaScrollPane);
		layout.putConstraint(javax.swing.SpringLayout.NORTH, fcScrollPane, 0, javax.swing.SpringLayout.NORTH, cp);

		cp.setLayout(layout);

		addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					System.exit(0);
				}
			}
		);
	}

	private void turnCostTrB_Scroll()
	{
		diag.getRoutingOptions().setTurnCost((byte)turnCostSl.getValue());
		turnCostLb.setText("Turn cost: " + turnCostSl.getValue());
	}

	private void crossingCostTrB_Scroll()
	{
		diag.getRoutingOptions().setCrossingCost((byte)crossingCostSl.getValue());
		crossingCostLb.setText("Crossing cost: " + crossingCostSl.getValue());
	}

	private void lengthCostTrB_Scroll()
	{
		diag.getRoutingOptions().setLengthCost((byte)lengthCostSl.getValue());
		lengthCostLb.setText("Length cost: " + lengthCostSl.getValue());
	}

	private void nodeVicCostTrB_Scroll()
	{
		diag.getRoutingOptions().setNodeVicinityCost((byte)nodeVicCostSl.getValue());
		nodeVicCostLb.setText("Node vicinity cost: " + nodeVicCostSl.getValue());
	}

	private void nodeVicSizeTrB_Scroll()
	{
		diag.getRoutingOptions().setNodeVicinitySize(nodeVicSizeSl.getValue());
		nodeVicSizeLb.setText("Node vicinity size: " + nodeVicSizeSl.getValue());
	}

	private void rerouteAllArrowsBtn_Click()
	{
		diag.routeAllLinks();
	}

	private JButton rerouteAllArrowsBtn;
	private JSlider nodeVicSizeSl;
	private JLabel nodeVicSizeLb;
	private JSlider nodeVicCostSl;
	private JLabel nodeVicCostLb;
	private JSlider lengthCostSl;
	private JLabel lengthCostLb;
	private JSlider crossingCostSl;
	private JLabel crossingCostLb;
	private JLabel turnCostLb;
	private JSlider turnCostSl;
	private JTextArea textBox1;
	private JScrollPane textAreaScrollPane;
	private JPanel optionPanel;

	private Diagram diag;
	private DiagramView diagView;
	private JScrollPane fcScrollPane;
	
	private static final long serialVersionUID = 1L;
}